<?php
/*
Plugin Name: Memo link generator
Plugin URI: http://mibai.tec.u-ryukyu.ac.jp/~oshiro/Programs/others/
Description: URLと次行の字下げタイトルからなるリンク情報を箇条書きのリンクリストに変換します。[mlink]...[/mlink]中に記述して下さい。
Version: 0.1.1
Author: Oshiro Naoki
Author URI: http://mibai.tec.u-ryukyu.ac.jp/~oshiro/
*/

if (class_exists('MemoLink')){
    new MemoLink();
}

class MemoLink {
    //classの決定
    var $tpl = '<div class="memo-link" {STYLE}>{CONTENT}</div>';
    var $blocks = array();

    var $indent=0;

    function __construct() {
	if (function_exists('add_filter')){
	    add_filter('the_content', array(&$this, 'first'), -1000);
	    add_filter('the_content', array(&$this, 'second'),  1000);
	}
    }
    
    function first($content) {
        $content = preg_replace('#(^|\n)\[mlink\](.*?)\[/mlink\]#sie', '$this->do_link2html(\'\\2\', $content);', $content);
        return $content;
    }
   
    function second($content) {
        $content = str_replace(array_keys($this->blocks), array_values($this->blocks), $content);
        $this->blocks = array();

        return $content;
    }

    function do_link2html($txt, $content) {
        $blockID = $this->getBlockID($content);

        $this->blocks[$blockID] = str_replace('{CONTENT}', $this->link2html($txt), $this->tpl);
     
        return $blockID;
    }

    function do_blocks($txt, $content) {
        $txt = str_replace(array("\\\"", "\\\'"), array("\"", "\'"), $txt);
        $blockID = $this->getBlockID($content);

        $this->blocks[$blockID] = '[mlink]' . $txt . '[/mlink]';

        return $blockID;
    }
    
    function getBlockID($content) {
        static $num = 0;

        do {
            ++$num;
            $blockID = "<div>::MEMOLINK_BLOCK_$num::</div>";
        } while(strpos($content, $blockID) !== false);

        return $blockID;
    }

    function link2html($content) {
	$spc_pre[0]=$spc="";
	$item=$next_item="<li>";
	$opt_t="";
	
	$text="";
	$lines=split("\n", $content);
	for ($l_no=0; $l_no<count($lines); $l_no++) {
	    $str=chop($lines[$l_no]);
	    if ($str=="") continue;
	    $str=str_replace('#\t#','        ',$str);
	    #&imgtag;
	    
	    #
	    # 表題処理
	    #
	    if (preg_match('#^\s*\[\d{4}/\d{2}/\d{2}\]#',$str)>0) {continue;} # 日付行は無視
	    if (preg_match('/^\s*#/',$str)>0) {continue;}                      # 行頭 '#' はコメントで無視
	    if (preg_match('/^(\s*)\|+/',$str)>0) { # 行頭が '|' なら連結表示
		$str=preg_replace('/^(\s*)\|+/','\1',$str);
		$item="||";   
	    }
	    if (preg_match('/^(\s*)((ftp|telnet|https?|file|href|mailto|news|javascript):.+)(\s*)$/',$str,$arr_str)==0) {
		# URL 'http:' などでなければ
		if (preg_match('/^(\s*)(.+)\s*$/',$str,$arr_str)>0) {
		    $spc=$arr_str[1];
		    $label=$arr_str[2];
		}
		if (preg_match('/^\s+/',$spc)>0) { # 行頭に空白があれば
		    $arr_spc=array(); # とりあえず空の配列を用意
		    if ($item!="||") {$text.=$this->indent_check($spc, $spc_pre);}
		    if (preg_match('/^\s*-+\s*$/',$label)>0) { # マイナス記号での横線指定の場合
			$text.="\n$spc<hr>\n";
		    } else {
			if ($item!="||") $text.="\n$spc";
			$text.="$item$label";
		    }
		    $item=$next_item; # [1997/06/23] OSHIRO Naoki. added 1 line.
		} else {
		    $text.="\n";
		    $ul=""; for ($i=0; $i<$this->indent; $i++) {$ul.="</ul>";} # インデント分<ul>を閉じる
		    if ($this->indent > 0) $text.="$spc_pre[1]" . $ul . "\n";
		    $item="<li>"; # [1997/06/25] OSHIRO Naoki. added 1 line
		    $text.="\n";
		    if (preg_match('/^-+\s*/',$str)>0) {
			$text.="<hr>";
		    } else  {
			if ($str!="") $text.="<b>$str</b>";
		    }
		    $spc_array=array();
		    $this->indent=0;
		}
		continue;
	    }
	    
	    #
	    # URL (http:) 処理
	    #
	    $spc=$arr_str[1]; $url=$arr_str[2]; # URL(HTTP) 取得
	    if ($item!="||") $text.=$this->indent_check($spc, $spc_pre);
	    #last if (eof);
	    
	    # 指定ファイルがカレントディレクトリにあるならば file:,http: を除く
	    $url=preg_replace('#^(file|http):([^/])#','\2',$url); 
	    
	    #
	    # ラベル処理
	    #
	    do {
		# 行頭 "#" である
		# 日付である
		# ならばラベルでない
		$str=$lines[++$l_no]; # 次の行を取得
		$str=preg_replace('#\t#','        ',$str);
		#&imgtag;
	    } while (preg_match('/^\s*#/',$str)>0 || preg_match('/^\s*\[\d{4}\/\d{2}\/\d{2}\]/',$str)>0); # テスト中
	    $keep=$str;
	    $next_item="<li>";
	    if (preg_match('/^(\s*)\|+/',$str, $arr_str)>0) { # 行頭が '|' なら連結表示
		$str=preg_match('/^(\s*)\|+/','\1',$str);
		$next_item="||";
	    }
	    if (preg_match('/^(\s*)http:/',$str)>0
		|| preg_match('/^$spc\s/',$str)!=0 # add "\s" for correction. [1997/06/29]
		|| ($next_item=="||")
		) { 
		# 'http:' である
		# 行頭の空白が $spc と同じ，または，より小さい
		# 行頭 "||" である
		# ならばラベルでない
		if ($item!="||") $text.="\n$spc";
		$text.="$item<a href=\"$url\">" . basename($url) . "</a>"; # $url のベースをラベルとして表示
		if ($item!="||") $text.=$this->indent_check($spc, $spc_pre);
		$item=$next_item;
		$next_item="<li>"; # [1998/03/27]
		#next if (eof);
		#$str=$keep;
		$l_no--; continue; #redo; # 取り込んだ行を処理しなおす
	    }
	    $str=preg_replace('/^\s*(.+)\s*/','\1',$str); # ラベル取得
	    if ($item!="||") $text.="\n$spc";
	    $text.="$item<a href=\"$url\">$str</a>";
	    if ($item!="||") $text.=$this->indent_check($spc, $spc_pre);
	    $item=$next_item;
	}
	$text.="\n";
	$ul=""; for ($i=0; $i<$this->indent; $i++) {$ul.="</ul>";}
	$text.="$ul\n";
	# 最後にラベルを閉じる
	$spc_pre=array();
	$this->indent=0;

	return $text;
    }
    
    function indent_check(&$spc, &$spc_pre) {
	$text="";
	for (;;) {
	    if ($spc==$spc_pre[0]) return $text; # 同じ深さなら何もしない．
	    if (preg_match("#^$spc_pre[0]\s+#",$spc)>0) {       # 行頭の空白を以前のものと比較して大きければ
		$text.="\n";
		$text.="$spc_pre[0]<ul>"; # <ul> を表示して
		array_unshift($spc_pre, $spc);
		$this->indent++;            # インデントレベルを上げる
		$spc=$spc_pre[0];
		return $text;
	    } else if ($this->indent>0) {     # そうでなく，さらにインデントレベルが正なら
		array_shift($spc_pre);
		$this->indent--;           #インデントレベルを下げ
		$text.="\n";
		$text.="$spc_pre[0]</ul>";  # </ul> で閉じる
	    }
	}
    }
    
    function imgtag($spc, $file, $label) {
	# 独自の img: タグを追加 [1997/11/28] OSHIRO Naoki.
	if (preg_match('/(\s*)img:(.+\.(jpg|gif))\s(.*)\s*$/',$label,$match)>0) {
	    chop;
	    $spc=$match[0]; $file=$match[1]; $label=$match[3];
	    if (preg_match('/^\s*$/',$label)>0) {
		$label=$file;
		$_="";
	    } else {
		$_=$label;
	    }
	    $_.="$spc<IMG SRC=\"$file\" BORDER=\"0\" ALT=\"$label\">";
	} else {
	    preg_replace('#img:(.+\.(p[pgb]m))\s#','<A HREF="\1">img:$1</A>');
	}
    }

    function basename($name) {
	if (preg_match('#([^/]+/)\s*$#',$name,$arr_name)>0) $name=$arr_name[0];
	if (preg_match('#([^/]+)(\.s?html?)?\s*$#',$name,$arr_name)>0) $name=$arr_name[0];
	$name=preg_replace('/\.s?html?/','',$name);
	return $name;
    }
}
?>
